==> prebid-adapter-2.3.2-sources.jar: main/com/particles/prebidadapter/Utils.kt <==
package com.particles.prebidadapter

import android.Manifest
import android.app.ActivityManager
import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.hardware.display.DisplayManager
import android.net.ConnectivityManager
import android.os.BatteryManager
import android.os.Build
import android.os.PowerManager
import android.text.TextUtils
import android.view.Display
import android.view.Surface
import android.view.WindowManager
import androidx.annotation.RequiresPermission
import androidx.core.app.ActivityCompat
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.ProcessLifecycleOwner
import com.particles.msp.util.Logger
import kotlin.reflect.KClass

fun createInstance(className: String): Any? {
    return try {
        val clazz = Class.forName(className)
        clazz.newInstance()
    } catch (e: ClassNotFoundException) {
        null
    } catch (e: InstantiationException) {
        null
    } catch (e: IllegalAccessException) {
        null
    }
}

fun <T : Any> createInstanceWithParameters(kClass: KClass<T>, vararg args: Any?): T? {
    val constructor = kClass.constructors.firstOrNull { it.parameters.size == args.size }
        ?: return null

    return constructor.call(*args)
}

fun getAppInstallTime(context: Context?): Long {
    val nonNullContext: Context = context ?: return 0

    return try {
        nonNullContext.packageManager.getPackageInfo(context.packageName, 0).firstInstallTime
    } catch (e: PackageManager.NameNotFoundException) {
        0
    }
}

internal fun getAvailableMemory(context: Context?): Long {
    val activityManager = context?.getSystemService(Context.ACTIVITY_SERVICE) as? ActivityManager

    if (activityManager == null) {
        Logger.info("getAvailableMemory: context is null or get ActivityManager failed.")
    }

    val memInfo = ActivityManager.MemoryInfo()
    activityManager?.getMemoryInfo(memInfo)
    return memInfo.availMem
}

internal fun getFontSize(context: Context?): String {
    return context?.let {
        val fontScale = it.resources.configuration.fontScale
        when {
            fontScale == 1.0f -> "m"
            fontScale < 1.0f -> "s"
            fontScale > 1.0f -> "l"
            else -> "unknown"
        }
    } ?: "unknown"
}

@RequiresPermission(Manifest.permission.ACCESS_NETWORK_STATE)
internal fun getIsLowDataMode(context: Context?): String {
    if (context == null) {
        Logger.info("getIsLowDataMode: context is null")
        return "unknown"
    }

    if (ActivityCompat.checkSelfPermission(
            context,
            Manifest.permission.ACCESS_NETWORK_STATE
        ) == PackageManager.PERMISSION_GRANTED
    ) {
        val connectivityManager =
            context.getSystemService(Context.CONNECTIVITY_SERVICE) as? ConnectivityManager

        if (connectivityManager == null) {
            Logger.info("getIsLowDataMode: get ConnectivityManager failed")
            return "unknown"
        } else {
            return if (connectivityManager.isActiveNetworkMetered) {
                "true"
            } else {
                "false"
            }
        }
    } else {
        Logger.info("getIsLowDataMode: no permission of ACCESS_NETWORK_STATE")
        return "unknown"
    }
}

internal fun getIsLowPowerMode(context: Context?): String {
    if (context == null) {
        Logger.info("getIsLowPowerMode: context is null")
        return "unknown"
    }

    val powerManager = context.getSystemService(Context.POWER_SERVICE) as? PowerManager
    if (powerManager == null) {
        Logger.info("getIsLowPowerMode: get PowerManager failed")
        return "unknown"
    } else {
        return if (powerManager.isPowerSaveMode) "true" else "false"
    }
}

internal fun getBatteryStatus(
    context: Context?,
    customParams: MutableMap<String, Any>
) {
    if (context == null) {
        Logger.info("getBatteryStatus: context is null")
        customParams[DEVICE_SIGNAL_BATTERY_LEVEL] = "unknown"
        customParams[DEVICE_SIGNAL_BATTERY_STATUS] = "unknown"
    } else {
        val batteryManager = context.getSystemService(Context.BATTERY_SERVICE) as? BatteryManager
        if (batteryManager == null) {
            Logger.info("getBatteryStatus: get BatteryManager failed")
            customParams[DEVICE_SIGNAL_BATTERY_LEVEL] = "unknown"
            customParams[DEVICE_SIGNAL_BATTERY_STATUS] = "unknown"
        } else {
            val batteryCapacity =
                batteryManager.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)
            customParams[DEVICE_SIGNAL_BATTERY_LEVEL] = "%.2f".format(batteryCapacity / 100f)

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                val batteryStatus =
                    batteryManager.getIntProperty(BatteryManager.BATTERY_PROPERTY_STATUS)
                customParams[DEVICE_SIGNAL_BATTERY_STATUS] = when (batteryStatus) {
                    BatteryManager.BATTERY_STATUS_CHARGING -> "charging"
                    BatteryManager.BATTERY_STATUS_DISCHARGING -> "unplugged"
                    BatteryManager.BATTERY_STATUS_FULL -> "full"
                    BatteryManager.BATTERY_STATUS_NOT_CHARGING -> "unknown"
                    BatteryManager.BATTERY_STATUS_UNKNOWN -> "unknown"
                    else -> "unknown"
                }
            } else {
                customParams[DEVICE_SIGNAL_BATTERY_STATUS] = "unknown"
            }
        }
    }
}

internal fun getOrientation(context: Context?): String {
    if (context == null) {
        Logger.info("getOrientation: context is null")
        return "unknown"
    } else {

        val display = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            val displayManager =
                context.getSystemService(Context.DISPLAY_SERVICE) as? DisplayManager
            if (displayManager == null) {
                Logger.info("getOrientation: get DisplayManager failed")
                return "unknown"
            }

            displayManager.getDisplay(Display.DEFAULT_DISPLAY)
        } else {
            val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as? WindowManager
            if (windowManager == null) {
                Logger.info("getOrientation: get WindowManager failed")
                return "unknown"
            }

            @Suppress("DEPRECATION")
            windowManager.defaultDisplay
        }

        val rotation = display.rotation
        val orientation = context.resources.configuration.orientation
        return when (rotation) {
            Surface.ROTATION_0 -> if (orientation == Configuration.ORIENTATION_PORTRAIT) "portrait" else "landscapeLeft"
            Surface.ROTATION_90 -> if (orientation == Configuration.ORIENTATION_LANDSCAPE) "landscapeLeft" else "portraitUpsideDown"
            Surface.ROTATION_180 -> if (orientation == Configuration.ORIENTATION_PORTRAIT) "portraitUpsideDown" else "landscapeRight"
            Surface.ROTATION_270 -> if (orientation == Configuration.ORIENTATION_LANDSCAPE) "landscapeRight" else "portrait"
            else -> "unknown"
        }
    }
}

internal fun getIsInForeground(): String {
    val isForeground = ProcessLifecycleOwner.get().lifecycle.currentState.isAtLeast(Lifecycle.State.STARTED)
    Logger.info("getIsInForeground: $isForeground")
    return if (isForeground) "true" else "false"
}

==> prebid-adapter-2.4.0-sources.jar: main/com/particles/prebidadapter/Utils.kt <==
package com.particles.prebidadapter

import android.Manifest
import android.app.ActivityManager
import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.hardware.display.DisplayManager
import android.net.ConnectivityManager
import android.os.BatteryManager
import android.os.Build
import android.os.PowerManager
import android.view.Display
import android.view.Surface
import android.view.WindowManager
import androidx.annotation.RequiresPermission
import androidx.core.app.ActivityCompat
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.ProcessLifecycleOwner
import com.particles.msp.util.Logger
import java.util.Locale
import java.util.TimeZone
import kotlin.math.abs
import kotlin.reflect.KClass

fun createInstance(className: String): Any? {
    return try {
        val clazz = Class.forName(className)
        clazz.newInstance()
    } catch (e: ClassNotFoundException) {
        null
    } catch (e: InstantiationException) {
        null
    } catch (e: IllegalAccessException) {
        null
    }
}

fun <T : Any> createInstanceWithParameters(kClass: KClass<T>, vararg args: Any?): T? {
    val constructor = kClass.constructors.firstOrNull { it.parameters.size == args.size }
        ?: return null

    return constructor.call(*args)
}

fun getAppInstallTime(context: Context?): Long {
    val nonNullContext: Context = context ?: return 0

    return try {
        nonNullContext.packageManager.getPackageInfo(context.packageName, 0).firstInstallTime
    } catch (e: PackageManager.NameNotFoundException) {
        0
    }
}

internal fun getAvailableMemory(context: Context?): Long {
    val activityManager = context?.getSystemService(Context.ACTIVITY_SERVICE) as? ActivityManager

    if (activityManager == null) {
        Logger.info("getAvailableMemory: context is null or get ActivityManager failed.")
    }

    val memInfo = ActivityManager.MemoryInfo()
    activityManager?.getMemoryInfo(memInfo)
    return memInfo.availMem
}

internal fun getDeviceTimezone(): String {
    val timeZone = TimeZone.getDefault()
    val offsetInMillis = timeZone.getOffset(System.currentTimeMillis())

    val hours = offsetInMillis / 3_600_000
    val minutes = (offsetInMillis % 3_600_000) / 60_000

    return String.format(Locale.US, "%+03d:%02d", hours, abs(minutes))
}

internal fun getFontSize(context: Context?): String {
    return context?.let {
        val fontScale = it.resources.configuration.fontScale
        when {
            fontScale == 1.0f -> "m"
            fontScale < 1.0f -> "s"
            fontScale > 1.0f -> "l"
            else -> "unknown"
        }
    } ?: "unknown"
}

@RequiresPermission(Manifest.permission.ACCESS_NETWORK_STATE)
internal fun getIsLowDataMode(context: Context?): String {
    if (context == null) {
        Logger.info("getIsLowDataMode: context is null")
        return "unknown"
    }

    if (ActivityCompat.checkSelfPermission(
            context,
            Manifest.permission.ACCESS_NETWORK_STATE
        ) == PackageManager.PERMISSION_GRANTED
    ) {
        val connectivityManager =
            context.getSystemService(Context.CONNECTIVITY_SERVICE) as? ConnectivityManager

        if (connectivityManager == null) {
            Logger.info("getIsLowDataMode: get ConnectivityManager failed")
            return "unknown"
        } else {
            return if (connectivityManager.isActiveNetworkMetered) {
                "true"
            } else {
                "false"
            }
        }
    } else {
        Logger.info("getIsLowDataMode: no permission of ACCESS_NETWORK_STATE")
        return "unknown"
    }
}

internal fun getIsLowPowerMode(context: Context?): String {
    if (context == null) {
        Logger.info("getIsLowPowerMode: context is null")
        return "unknown"
    }

    val powerManager = context.getSystemService(Context.POWER_SERVICE) as? PowerManager
    if (powerManager == null) {
        Logger.info("getIsLowPowerMode: get PowerManager failed")
        return "unknown"
    } else {
        return if (powerManager.isPowerSaveMode) "true" else "false"
    }
}

internal fun getBatteryStatus(
    context: Context?,
    customParams: MutableMap<String, Any>
) {
    if (context == null) {
        Logger.info("getBatteryStatus: context is null")
        customParams[DEVICE_SIGNAL_BATTERY_LEVEL] = "unknown"
        customParams[DEVICE_SIGNAL_BATTERY_STATUS] = "unknown"
    } else {
        val batteryManager = context.getSystemService(Context.BATTERY_SERVICE) as? BatteryManager
        if (batteryManager == null) {
            Logger.info("getBatteryStatus: get BatteryManager failed")
            customParams[DEVICE_SIGNAL_BATTERY_LEVEL] = "unknown"
            customParams[DEVICE_SIGNAL_BATTERY_STATUS] = "unknown"
        } else {
            val batteryCapacity =
                batteryManager.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)
            customParams[DEVICE_SIGNAL_BATTERY_LEVEL] = "%.2f".format(batteryCapacity / 100f)

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                val batteryStatus =
                    batteryManager.getIntProperty(BatteryManager.BATTERY_PROPERTY_STATUS)
                customParams[DEVICE_SIGNAL_BATTERY_STATUS] = when (batteryStatus) {
                    BatteryManager.BATTERY_STATUS_CHARGING -> "charging"
                    BatteryManager.BATTERY_STATUS_DISCHARGING -> "unplugged"
                    BatteryManager.BATTERY_STATUS_FULL -> "full"
                    BatteryManager.BATTERY_STATUS_NOT_CHARGING -> "unknown"
                    BatteryManager.BATTERY_STATUS_UNKNOWN -> "unknown"
                    else -> "unknown"
                }
            } else {
                customParams[DEVICE_SIGNAL_BATTERY_STATUS] = "unknown"
            }
        }
    }
}

internal fun getOrientation(context: Context?): String {
    if (context == null) {
        Logger.info("getOrientation: context is null")
        return "unknown"
    } else {

        val display = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            val displayManager =
                context.getSystemService(Context.DISPLAY_SERVICE) as? DisplayManager
            if (displayManager == null) {
                Logger.info("getOrientation: get DisplayManager failed")
                return "unknown"
            }

            displayManager.getDisplay(Display.DEFAULT_DISPLAY)
        } else {
            val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as? WindowManager
            if (windowManager == null) {
                Logger.info("getOrientation: get WindowManager failed")
                return "unknown"
            }

            @Suppress("DEPRECATION")
            windowManager.defaultDisplay
        }

        val rotation = display.rotation
        val orientation = context.resources.configuration.orientation
        return when (rotation) {
            Surface.ROTATION_0 -> if (orientation == Configuration.ORIENTATION_PORTRAIT) "portrait" else "landscapeLeft"
            Surface.ROTATION_90 -> if (orientation == Configuration.ORIENTATION_LANDSCAPE) "landscapeLeft" else "portraitUpsideDown"
            Surface.ROTATION_180 -> if (orientation == Configuration.ORIENTATION_PORTRAIT) "portraitUpsideDown" else "landscapeRight"
            Surface.ROTATION_270 -> if (orientation == Configuration.ORIENTATION_LANDSCAPE) "landscapeRight" else "portrait"
            else -> "unknown"
        }
    }
}

internal fun getIsInForeground(): String {
    val isForeground = ProcessLifecycleOwner.get().lifecycle.currentState.isAtLeast(Lifecycle.State.STARTED)
    Logger.info("getIsInForeground: $isForeground")
    return if (isForeground) "true" else "false"
}
